<?php

namespace Ems\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\DocumentoService;

class DocumentoController {
    /** @var DocumentoService */
    protected $documentoService;

    public function __construct(
        DocumentoService $documentoService
    ) {
        $this->documentoService = $documentoService;
    }

    public function getDocumentosVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];

            $nrCategoria = isset($row['NRCATEGDOCUMENTO']) ?
                (string) $row['NRCATEGDOCUMENTO'] :
                null;

            $documentos = $this->documentoService->getDocumentosVinculo(
                $nrVinculo, $nrCategoria
            );

            $response->addDataSet(new DataSet('getDocumentosVinculo', $documentos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getCategoriasComDocumentoVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];

            $categorias = $this->documentoService->getCategoriasComDocumentoVinculo(
                $nrVinculo
            );

            $response->addDataSet(new DataSet('getCategoriasComDocumentoVinculo', $categorias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getCategorias(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $categorias = $this->documentoService->getCategorias();

            $response->addDataSet(new DataSet('getCategorias', $categorias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTipos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipos = $this->documentoService->getTipos();

            $response->addDataSet(new DataSet('getTipos', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function downloadDocumento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrAnexo = (int) $row['NRANEXO'];

            $file = $this->documentoService->downloadDocumento($nrAnexo);

            $response->addDataSet(new DataSet('downloadDocumento', $file));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveDocumento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->documentoService->saveDocumento($row);

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('saveDocumento', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveDocumentos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->documentoService->saveDocumentos($row);

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('saveDocumento', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveDocumentosQrCode(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->documentoService->saveDocumentos(
                $row,
                DocumentoService::CATEG_QR_CODE,
                DocumentoService::TIPO_FOTO
            );

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('saveDocumentosQrCode', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveDocumentoAssinatura(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->documentoService->saveDocumentoAssinatura(
                $row,
                DocumentoService::CATEG_DEMONSTRATIVO_PONTO,
                DocumentoService::TIPO_FOTO
            );

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('saveDocumentoAssinatura', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function saveAssinaturaVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $this->documentoService->saveDocumentos(
                $row,
                DocumentoService::CATEG_ASSINATURA,
                DocumentoService::TIPO_DOCUMENTO
            );

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('saveAssinaturaVinculo', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getAssinaturaByVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];

            $nomeCategoria = DocumentoService::CATEG_ASSINATURA;
            $nomeTipo = DocumentoService::TIPO_FOTO;

            $assinatura = $this->documentoService->getDocumentosVinculo($nrVinculo, null, $nomeCategoria, $nomeTipo);

            $assinatura = is_array($assinatura) && isset($assinatura[0]) ? $assinatura[0] : $assinatura;

            $response->addDataSet(new DataSet('getAssinaturaByVinculo', $assinatura));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function deleteDocumento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrAnexo = (int) $row['NRANEXO'];

            $this->documentoService->deleteDocumento($nrAnexo);

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('deleteDocumento', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
