<?php

namespace Ems\Controllers;

use Ems\Services\EstruturaService;
use Ems\Util\ArrayUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class EstruturaController {
    /** @var EstruturaService */
    protected $estruturaService;

    public function __construct(
        EstruturaService $estruturaService
    ) {
        $this->estruturaService = $estruturaService;
    }

    public function getEstruturasGerenciais(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $cdOperador = (string) $row['CDOPERADOR'];

            $nrEstruturas = $this->estruturaService->
                getEstruturasGerenciais($cdOperador);

            $estruturas = array_map(function($nrEstrutura) {
                $infoEstrutura = $this->estruturaService->
                    getNomeETipoEstrutura($nrEstrutura);

                return $infoEstrutura;
            }, $nrEstruturas);

            $this->estruturaService->insertEstruturaTodas(
                $cdOperador, $estruturas
            );

            $response->addDataSet(new DataSet('getEstruturasGerenciais', $estruturas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getNrEstruturasGerenciais(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $cdOperador = (string) $row['CDOPERADOR'];

            $nrEstruturas = $this->estruturaService->
                getEstruturasGerenciais($cdOperador);

            $response->addDataSet(new DataSet('getNrEstruturasGerenciais', $nrEstruturas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getEstruturasInferiores(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrEstrutura = (int) $row['NRESTRUTURAM'];

            $nrEstruturas = $this->estruturaService->
                getTodasEstruturasInferiores($nrEstrutura, true);

            $response->addDataSet(new DataSet('getEstruturasInferiores', $nrEstruturas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function getEstrutura(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nmTipoEstrutura = $row['NMTIPOESTRUTURA'] ?? null;

            $estruturas = $this->estruturaService->
                getEstrutura($nmTipoEstrutura);

            $response->addDataSet(new DataSet('getEstrutura', $estruturas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getEstruturas(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrTipoEstrutura = $row['NMTIPOESTRUTURA'] ?? null;
            $nrorg = $row['NRORG'] ?? null;

            $estruturas = $this->estruturaService->getEstruturas($nrTipoEstrutura, $nrorg);

            $response->addDataSet(new DataSet('getEstruturas', $estruturas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getEstruturaAdicional(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $estruturas = $this->estruturaService->
                getEstruturaAdicional();
            
            $response->addDataSet(new DataSet('getEstruturaAdicional', $estruturas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getAniversariantesEstrutura(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrEstrutura = (int) $row['NRESTRUTURAM'];

            $aniversariantes = $this->estruturaService->getAniversariantesEstrutura($nrEstrutura);

            $response->addDataSet(new DataSet('getAniversariantesEstrutura', $aniversariantes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
