<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

use Ems\Services\MinhaHistoriaService;
use Ems\Util\Utils;

class MinhaHistoriaController {
    protected $minhaHistoriaService;

    protected $bloqDiaIgualSoli = ['3', '9', '21', '22', '27'];

    public function __construct(MinhaHistoriaService $minhaHistoriaService) {
        $this->minhaHistoriaService = $minhaHistoriaService;
    }

    public function solicitaAlteracao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            if ($row["NRTIPOSOLICITACAO"] == 16) {
                $nrSolicitacao = $this->minhaHistoriaService->solicitaAlteracaoComunica($row);
            } else {
                $nrSolicitacao = $this->minhaHistoriaService->solicitaAlteracao($row);
            }

            $dataset = [
                'success' => true,
                'nrsolicitacao' => $nrSolicitacao,
            ];

            $response->addDataSet(new DataSet('solicitaAlteracao', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSolicitacaoFile(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao = (int) $row['NRSOLICITACAO'];

            $file = $this->minhaHistoriaService->
                getSolicitacaoFile($nrSolicitacao);

            $response->addDataSet(new DataSet('getSolicitacaoFile', $file));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getSolicitacoesDia(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrSoliAlte = false;
            $dsSoliAlte = '';
            $msg = null;
            
            if(in_array($row["NRTIPOSOLICITACAO"], $this->bloqDiaIgualSoli)) {
                $existeSolicitacao = $this->minhaHistoriaService->verificaBloqueioSolicitacao($row);
                if($existeSolicitacao && isset($existeSolicitacao[0]['NRSOLICITACAO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['NRSOLICITACAO'];
                    $dsSoliAlte = "uma solicitação";
                    $dsFimSoliAlte = "deste tipo para o dia selecionado. Favor selecionar outro dia";
                } else if($existeSolicitacao && isset($existeSolicitacao[0]['ALTERACAO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['ALTERACAO'];
                    $dsSoliAlte = "uma alteração";
                    $dsFimSoliAlte = "deste tipo para o dia selecionado. Favor selecionar outro dia";
                } else if($existeSolicitacao && isset($existeSolicitacao[0]['BENEFICIO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['BENEFICIO'];
                    $dsSoliAlte = "um benefício";
                    $dsFimSoliAlte = "deste tipo em ativo. Favor verificar a vigência";
                } else if($existeSolicitacao && isset($existeSolicitacao[0]['EVENTO'])) {
                    $nrSoliAlte = $existeSolicitacao[0]['EVENTO'];
                    $dsSoliAlte = "um evento relacionado";
                    $dsFimSoliAlte = "deste tipo em ativo. Favor verificar a vigência";
                }
            }
            
            if($nrSoliAlte) {
                $msg = 'Já existe ' . $dsSoliAlte . ' (N° '. $nrSoliAlte . ') ' . $dsFimSoliAlte;
            }
            
            $dataset = [
                'success' => true,
                'mensagem' => $msg,
            ];
            
            $response->addDataSet(new DataSet('getSolicitacoesDia', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

}
