<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Util\{ DateUtil, Repositories };
use Ems\Util\Utils;

class MinhaHistoriaRepository {
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function insereSolicitacao($nrsolicitacao, $nrtiposolicitacao, $nrvinculom, $nrorg, $cdoperador, $dsobservacao, $dspath = null) {

        $nrmodeloaprovacao = $this->entityManager->getRepository(Repositories::GPE_SOLICITACAO)->getModeloSolicitacaoBeforeInsert(
            $nrorg, $nrtiposolicitacao, $nrvinculom, Utils::getParamters()->getDtmescomptencDiaCorrente()
        );

        $sql = <<<SQL
            INSERT INTO GPE_SOLICITACAO
                (NRORG, NRSOLICITACAO, NRTIPOSOLICITACAO, NRVINCULOM, DTSOLICITACAO, DSOBSSOLICITACAO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTULTATU, CDOPERULTATU, NRORGULTATU, DSPATH, NRMODELOAPROVSOLI)
            VALUES
                (:NRORG, :NRSOLICITACAO, :NRTIPOSOLICITACAO, :NRVINCULOM, :DTSOLICITACAO, :DSOBSSOLICITACAO, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO, :DTULTATU, :CDOPERULTATU, :NRORGULTATU, :DSPATH, :NRMODELOAPROVSOLI)
SQL;

        return $this->connection->prepare($sql)
            ->execute(array(
                "NRORG"             => $nrorg,
                "NRSOLICITACAO"     => $nrsolicitacao,
                "NRTIPOSOLICITACAO" => $nrtiposolicitacao,
                "NRVINCULOM"        => $nrvinculom,
                "DTSOLICITACAO"     => DateUtil::getDataAtual(true)->format('d/m/Y'),
                "DSOBSSOLICITACAO"  => $dsobservacao,
                "DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador,
                "DTULTATU"          => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdoperador,
                "DSPATH"            => $dspath,
                "NRMODELOAPROVSOLI" => $nrmodeloaprovacao,
            )
        );
    }
    
    public function insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, $dschaveitemsoli, $dslabelitemsoli, $dsvaloritemsoli, $nrorg, $cdoperador) {

        $query = 'INSERT INTO GPE_ITEMSOLICITACAO
                      (NRORG, NRITEMSOLICITACAO, NRSOLICITACAO, DSCHAVEITEMSOLI, DSLABELITEMSOLI, DSVALORITEMSOLI, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
                    VALUES
                      (:NRORG, :NRITEMSOLICITACAO, :NRSOLICITACAO, :DSCHAVEITEMSOLI, :DSLABELITEMSOLI, :DSVALORITEMSOLI, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO)';

        return $this->connection->prepare($query)
            ->execute(array(
                "NRORG"             => $nrorg,
                "NRITEMSOLICITACAO" => $nritemsolicitacao,
                "NRSOLICITACAO"     => $nrsolicitacao,
                "DSCHAVEITEMSOLI"   => $dschaveitemsoli,
                "DSLABELITEMSOLI"   => $dslabelitemsoli,
                "DSVALORITEMSOLI"   => $dsvaloritemsoli,
                "DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                "NRORGINCLUSAO"     => $nrorg,
                "CDOPERINCLUSAO"    => $cdoperador,
            )
        );
    }

    public function verificaBloqueioSolicitacao($params) {

        $sql = <<<SQL
        SELECT 
            SO.NRSOLICITACAO
        FROM 
            GPE_SOLICITACAO SO
        JOIN GPE_ITEMSOLICITACAO ISO
            ON ISO.NRORG = SO.NRORG
                AND ISO.NRSOLICITACAO = SO.NRSOLICITACAO
        WHERE 
            SO.NRVINCULOM = :P_NRVINCULOM 
            AND SO.NRORG = :P_NRORG
            AND ISO.DSLABELITEMSOLI = :P_DSLABELITEMSOLI
            AND ISO.DSVALORITEMSOLI = :P_DSVALORITEMSOLI
            AND SO.NRTIPOSOLICITACAO = :P_NRTIPOSOLICITACAO
            AND SO.IDAPROVACAOSEC <> 'R'
SQL;

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function verificaBloqueioAlteracao($tabela, $campo, $campoData, $params) {

        $sql = <<<SQL
        SELECT 
            $campo AS ALTERACAO
        FROM 
            $tabela
        WHERE 
            NRVINCULOM = :P_NRVINCULOM 
            AND NRORG = :P_NRORG
            AND $campoData = :P_CAMPODATA
SQL;

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function verificaBeneficiario($params) {

        $sql = <<<SQL
        SELECT 
            NRBENEFICIARIO AS BENEFICIO
        FROM 
            VAB_BENEFICIARIO
        WHERE 
            NRVINCULOM = :P_NRVINCULOM 
            AND NRORG = :P_NRORG
            AND NRBENEFICIOM = :P_NRBENEFICIOM
            AND (DTCANCBENEFICIARIO IS NULL 
                   OR DTCANCBENEFICIARIO = (SELECT 
                                              MAX(B.DTCANCBENEFICIARIO) 
                                            FROM 
                                              VAB_BENEFICIARIO B 
                                            WHERE 
                                              B.NRORG = NRORG 
                                              AND B.NRVINCULOM = NRVINCULOM 
                                              AND B.DTCANCBENEFICIARIO >= :P_DTATUAl))
SQL;

        $query = $this->connection->prepare($sql);
        $query->execute($params);

        return $query->fetchAll();
    }

    public function verificaEventoAtivo($params) {
        $sql = <<<SQL
        SELECT 
            NRVINCEVENTO AS EVENTO
        FROM 
            FPA_VINCEVENTO
        WHERE 
            NRORG = :P_NRORG
            AND NRVINCULOM = :P_NRVINCULOM
            AND NREVENTOM = :P_NREVENTOM
            AND (DTFIMVIGENCIA >= :P_DTATUAL OR DTFIMVIGENCIA IS NULL)
  
SQL;
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        return $query->fetchAll();
    }

    
}