<?php

namespace HCM\FPA\Calculos\Rescisao;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\FPA\Services\CalculoFolhaService;
use HCM\FPA\Entity\FpagozoferiasEntity;
use HCM\FPA\Entity\FparesumoctblEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaTpvincprocessoEntity;
use HCM\FPA\Entity\FpaDireitotpdemisEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\FPA\Calculos\Rescisao\CalculoRescisao;
use HCM\FPA\Calculos\Geral\Dependencia;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Calculos\Geral\Preparacao;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use Teknisa\Model\Gestaopessoas\Repositories\FpaCalculoconsumer;

class PercorreRescisao{

    private $_parametros;
    private $_nrOrgPadrao;
    private $_nrOrgTrabalho;
    private $_nrEstruturaTrab;
    private $_idDebug;
    private $_ambienteSalarioOperador;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;
    private $_mesCompetencia;
    private $_nrTipoCalculo;
    private $_simulacao;
    private $_retrocesso;
    private $_parcial;
    private $_dtPagamento;
    private $_ultPagamento;
    private $_dt_inicio;
    private $_dt_fim;
    private $_dsInicio;
    private $_dsFim;
    private $_id_retrocesso;
    private $_id_fazDifComp;
    private $_dsOrdem;
    private $_tipo_calculo;
    private $_calculo_folha;
    private $_tp_calc_mov;
    private $_vinculom;
    private $_tpVinculoProcesso;
    private $_tpRescisaoDireito;
    private $_pessoa;
    private $_dependencia;
    private $_formulario;
    private $_msg_idstatus;
    private $_msg_idabort;
    private $_msg_mensagens;
    private $_cadGozoFerias;

    public function __construct($entityManager, $hcmParameters, $dataSet){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->dataSet = $dataSet;
        $this->_nrTipoCalculo = $dataSet['TP_CALCULO'];
        $this->_parametros = $hcmParameters;
        $this->_nrOrgTrabalho = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = $this->hcmParameters->getDtCompetencia();
        $this->_nrTipoModalidadeCalculo = $this->hcmParameters->getTipoModalidadeCalculo();
        $this->_nrOcorrenciaCalculo = $this->hcmParameters->getNrOcorrenciaCalculo();
        $this->_buscaDadosFormularioCalculo($dataSet);
        $this->_retrocesso = false;
        $this->_parcial = false;
        $this->_cadGozoFerias = false;
        $this->_cancelamento = false;

        $this->tpProcessamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        switch ($this->tpProcessamento){
            case FPA::PROC_SIMULACAO :
                $this->_simulacao = true; break;
            case FPA::PROC_CALCULO :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true; break;
            case FPA::PROC_CALCULO_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_CADGOZOFERIAS :
                $this->_cadGozoFerias = true; break; 
            case FPA::PROC_CANCELAMENTO :
                $this->_cancelamento = true; break; 
            case FPA::PROC_RECALCPATRONAL :
                $this->_simulacao = false; break;
            default:
                throw new \Exception('Tipo de processo '.$this->tpProcessamento.' invalido');
        }

        Inicializa::Container($entityManager, $hcmParameters, $this->_dtPagamento);
        Inicializa::carregaVariaveis();
        
        $this->_vinculom = TkDI::getContainer()->vinculom;
        $this->_tipo_calculo = TkDI::getContainer()->tipo_calculo;
        $this->_calculo_folha = TkDI::getContainer()->calculo_folha;
        $this->_pessoa = TkDI::getContainer()->pessoa;
        $this->_tp_calc_mov = TkDI::getContainer()->tp_calc_mov;
        $this->_dependencia = new Dependencia($entityManager, $hcmParameters);
        $this->_tpVinculoProcesso = new FpaTpvincprocessoEntity($entityManager, $hcmParameters);
        $this->_tpRescisaoDireito = new FpaDireitotpdemisEntity($entityManager, $hcmParameters);
        $this->objTipoCalculo = new FpatipocalculoEntity($entityManager, $hcmParameters);

    }

    public function percorre(){
       
        $objTpCalc = $this->objTipoCalculo->retornaTipoCalculo($this->_nrTipoCalculo);
        
        try {
            $this->_tp_calc_mov->retornaTipoCalculoMovimento($this->_nrTipoCalculo);  ///
        } catch (\Exception $e){
            throw $e;
        }
        $this->_tp_calc_mov->atribuiNrCalculoFolha(
            $this->_nrOrgTrabalho, $this->_calculo_folha, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo,
            $this->_nrOcorrenciaCalculo
        );
        
        $this->_tpRescisaoDireito->carregaTipoDireitoPorTipoDemissao(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );

        $this->_tpVinculoProcesso->carregaTipoVinculoPorProcesso(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );
        
        $vinculos = $this->_vinculom->retornaVinculosPorTipoCalculo($this->_nrTipoCalculo,$this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim,
            $this->_dt_inicio, $this->_dt_fim, $this->_dsOrdem, $this->_nrEstruturaTrab);
        
        $qtd_vinculo = count($vinculos);
        
        if($qtd_vinculo > 1 && $this->_retrocesso && !$this->dataSet['RETROCESSO_CONFIRMADO']){
            return array('CONFIRMACAO' => true, 'QTDVINCULO' => $qtd_vinculo);
        }
        
        if ($qtd_vinculo > 0) {
            
            // cria registro no calculoconsumer
            $nrcalculoconsumer = $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->insereCalculoConsumer($this->_nrOrgTrabalho, $qtd_vinculo, $this->hcmParameters, $this->_dsOrdem, $this->_dsInicio, $this->_dsFim);
            $calcConsumer = \Setforms2\Shared\Registry::getEntityManager()->getRepository(Repositories::FPA_CALCULOCONSUMER);
            // retorna registro do calculoconsumer da fila e seta a data hora de inicio
            $fpaCalculoConsumer = $calcConsumer->findOneBy(array('nrcalculoconsumer' => $nrcalculoconsumer, 'nrorg' => $this->_nrOrgTrabalho));
            
            if ($fpaCalculoConsumer->getIdsituacao() == FpaCalculoconsumer::IDSITUACAO_PENDENTE)
                $fpaCalculoConsumer->setIdsituacao(FpaCalculoconsumer::IDSITUACAO_ANDAMENTO);
            if (is_null($fpaCalculoConsumer->getDthrinicio()))
                $fpaCalculoConsumer->setDthrinicio(DateUtil::getDataAtual());
                
            $resultado = array();
            $contador = 0;
            $idstatus = MsgCalculo::ID_PENDENTE;
            $idabort = false;

			$calculo = new CalculoRescisao($this->entityManager, $this->hcmParameters, $this->dataSet, $this->_nrTipoCalculo, $this->_simulacao, $this->_parcial);
            
            $preparacao = new Preparacao($this->entityManager, $this->hcmParameters, $this->dataSet, $this->_nrTipoCalculo, false, false);
            
            $obj_tp_calc_mov = current($this->_tp_calc_mov->retornaPosicaoTpMov($this->_tp_calc_mov->retornaTpMovPrincipal()));
            $nrcalcprincipal = $obj_tp_calc_mov->getNrCalcFolha();
            
            $resultadoCalcular = null;
                
            foreach ($vinculos as $key => $vinculo) {
            
                if($key > 9){
                    MemoriaCalculo::getInstance()->setExecutaMemoriaCalculo('N');
                }
                
                MsgCalculo::getInstance()->setDados($nrcalcprincipal, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'ERROSCALCULO');

                if($contador >= 10 && $this->_simulacao==true){
                    MsgCalculo::getInstance()->adicionaMensagem("Simulado e demonstrado apenas os 10 primeiros vínculos da lista. Para calcular mais vínculos será necessário gravar.");
                    break;
                }
                $listaProcessos = $this->_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());
                    
                if (is_null($listaProcessos)) {
                    MsgCalculo::getInstance()->adicionaMensagem('Tipo de vínculo não possui processos de cálculo');
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    continue;
                }
                
                if (in_array(FpaProcessoEntity::RESCISAO, $listaProcessos) === false){
                    continue;
                } else {
                    $listaDireitos = $this->_tpRescisaoDireito->getItem($vinculo->getNrtpdemissao());
                    $calculo->setListaDireitos($listaDireitos);
                    if ($vinculo->getNrtpdemissao() == null){
                        MsgCalculo::getInstance()->adicionaMensagem('Vínculo sem tipo de demissão.');
                    }
                }  
                    
                if ($this->_retrocesso === false && $this->_cancelamento === false) {
                    try {
                        $ferias = $this->verificaFeriasPosteriorRescisao($vinculo);
                        
                        if($ferias){
                            if (!$this->finalizaMensagens()) {
                                continue;
                            }    
                            continue;
                        }
                        
                        //Não preparar quando for RESCISÃO COMPLEMENTAR e for RECALCULO DA CONTRIBUIÇÃO PATRONAL DO INSS
                        if(is_null($this->dataSet['TP_PROCESSAMENTOAUX'])){
                            $preparacao->preparar($vinculo, $this->_tp_calc_mov, $contador);
                        }
                    } catch (\Exception $e) {
                        MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
                        if (!$this->finalizaMensagens()) {
                            continue;
                        }
                        continue;
                    }
                    $this->_dependencia->analisaDependencia($vinculo);
                    
                    try {
                        $calculo->setListaProcessos($listaProcessos);
                        $resultadoCalcular[] = $calculo->calculaRescisao($vinculo, $this->_tp_calc_mov, $contador);
                    } catch (\Exception $e) {
                        MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
                        if (!$this->finalizaMensagens()) {
                            continue;
                        }
                        continue;
                    }
                }else if($this->_cancelamento === true){
                    try {
                        $calculo->cancelamento($vinculo, $this->_tp_calc_mov);
                    } catch (\Exception $e){
                        throw new \Exception($e->getMessage());
                    }
                } else {
                    try {
                        $calculo->retroceder($vinculo, $this->_tp_calc_mov);
                    } catch (\Exception $e){
                        throw new \Exception($e->getMessage());
                    }
                }
                if (!$this->finalizaMensagens()) {
                    continue;
                }
                
                $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->addNrtotprocessvinc($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
                
                $contador++;
            }
            if ($this->_msg_idabort === false) {
                try {
                    $fpacalculofolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrcalculofolha' => $nrcalcprincipal, 'nrorg' => $this->_nrOrgTrabalho));
                    if (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)) {
                        $fpacalculofolha->setIdstatusproc('E');
                    } else {
                        $fpacalculofolha->setIdstatusproc('C');
                    }
                    $fpacalculofolha->persist();
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                } catch (\Exception $e) {
                    \Setforms2\Shared\Registry::getEntityManager()->rollback();
                    throw $e;
                }
            }
            
            \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();
    
            $this->entityManager->getRepository(Repositories::FPA_CALCULOCONSUMER)->concluirCalculoConsumer($this->_nrOrgTrabalho, $fpaCalculoConsumer->getNrcalculoconsumer());
            $fpaCalculoConsumer->setNrcalculofolha($nrcalcprincipal);
            //$fpaCalculoConsumer->setIderro(($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA));
            $fpaCalculoConsumer->setDscalculoparams('Cálculo do tipo '.$this->_nrTipoCalculo.' realizado pela rotina sequencial (Não utilizando Lote), Tipo de Processamento:'.$this->tpProcessamento);
            \Setforms2\Shared\Registry::getEntityManager()->persist($fpaCalculoConsumer);
            \Setforms2\Shared\Registry::getEntityManager()->flush();
            \Setforms2\Shared\Registry::getEntityManager()->commit();
            
            return array(
                'DEMONSTRATIVO' => $resultadoCalcular,
                'CALCULO' => $nrcalcprincipal,
                'MENSAGENS' => ($this->_msg_idabort === true ? $this->_msg_mensagens : null),
                'ERRO' => ($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO),
                'ABORT' => $this->_msg_idabort
            );
        } else {
            return null;
        }
    }
    
    public function verificaFeriasPosteriorRescisao($vinculo){
        $dtRescisao = DateUtil::getDataDeString($vinculo->getDtrescisaovinc());
        $gozoferiasEntity = new FpagozoferiasEntity($this->entityManager, $this->hcmParameters);
        $gozoferiasEntity->retornaGozoFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());
        $gozosFerias = $gozoferiasEntity->getItems();
        if($gozosFerias){
            foreach ($gozosFerias as $gozo) {
                $dtIni = DateUtil::getDataDeString($gozo->getDtinigozoferias()); 
                $dtFim = DateUtil::getDataDeString($gozo->getDtfimgozoferias()); 
                if($dtFim > $dtRescisao || $dtFim->format('d/m/Y') == $dtRescisao->format('d/m/Y')){
                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo possui gozo de ferias posterior a rescisão: '.$dtIni->format('d/m/Y').' '.$dtFim->format('d/m/Y'), MsgCalculo::ERRO);
                    return true;
                }
            }
        }
        return false;
    }
    
    private function _buscaDadosFormularioCalculo($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dt_inicio = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOINI'],0, 10));
        $this->_dt_fim = DateUtil::getDataDeString(substr($dataSet['DTRESCISAOFIM'], 0, 10));
    }

    private function finalizaMensagens() {
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            $this->_msg_mensagens = $msgcalculo['MENSAGENS'];
            return false;
        } else {
            MsgCalculo::getInstance()->salvaMensagens();
        }
        MsgCalculo::getInstance()->clear();
        return true;
    }
}