<?php

    namespace HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico;

    use Doctrine\ORM\EntityManager;
    use HCM\Util\HCMParameters;
    use HCM\Util\FPA;
    use HCM\Util\DateUtil;
    use HCM\Util\Repositories;
    use HCM\Util\MsgCalculo;
    use HCM\FPA\Entity\TipohierarqorgEntity;
    use HCM\FPA\Entity\HierarquiaorgEntity;
    use HCM\FPA\Entity\FpaitecalcfolhaEntity;
    use HCM\FPA\Entity\FpaHoleritemagnetEntity;
    use HCM\FPA\Entity\FpaeventomEntity;
    use HCM\Geral\Entity\GpepessoaEntity;
    use HCM\Geral\Entity\GpevinculomEntity;
    use HCM\Geral\Entity\GpemovimentacaoEntity;
    use HCM\Geral\Entity\GpeOcupacaomEntity;
    use HCM\Geral\Entity\ContcorrparcEntity;
    use HCM\Geral\Entity\EstruturamEntity;
    use HCM\FPA\Calculos\Geral\Estruturacao;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\santander\HoleriteSantander;
    use HCM\FPA\ObrigacaoLegal\Demonstrativo_Eletronico\RegistroHolerite;
    

class Holerite{
    
    const TIPO_TRANSFERENCIA_LEGAL = '1';

    public function __construct($nrorg, $nrorgPadrao, $mesCompetencia, $holeriteMagnetico, $formulario, $nrCalculoFolha,EntityManager $entityManager,HcmParameters $hcmParameters){

        $this->nrorg = $nrorg;
        $this->nrorgPadrao = $nrorgPadrao;
        $this->mesCompetencia = $mesCompetencia;
        $this->nrCalculoFolha = $nrCalculoFolha;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;

        $this->dtInicio = $mesCompetencia;

        $this->dtFim = DateUtil::adicionaIntervalo($mesCompetencia,1,DateUtil::MES);
        $this->dtFim = DateUtil::subtraiIntervalo($this->dtFim,1);

        $this->itemCalculoFolha = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters);
        $this->evento = new FpaeventomEntity($this->entityManager, $this->hcmParameters);
        $this->objPessoa = new GpepessoaEntity($this->entityManager, $this->hcmParameters);
        $this->vinculo = new GpevinculomEntity($this->entityManager, $this->hcmParameters);
        $this->movimentacao = new GpemovimentacaoEntity($this->entityManager, $this->hcmParameters);
        $this->ocupacao = new GpeOcupacaomEntity($this->entityManager, $this->hcmParameters);

        $this->contaCorrenteParceiro = new ContcorrparcEntity($this->entityManager, $this->hcmParameters);

        $this->holeriteMagnetico = $holeriteMagnetico;

        $this->holeriteSantander = new HoleriteSantander($formulario,$this->hcmParameters,$this->entityManager,$this->holeriteMagnetico);
        $this->registroHolerite = new RegistroHolerite();

        $this->estruturacao = new Estruturacao($this->entityManager,$this->hcmParameters);
        $this->estruturamEntity = new EstruturamEntity($this->entityManager,$this->hcmParameters);


        $this->inicializaVariaveis($formulario);
    }


    private function inicializaVariaveis($formulario){

        $this->estruturacao->carregaHierarquias($formulario->getNrhierarqorg());
        $this->estruturacao->carregaTipoHierarquia($formulario->getNrhierarqorg());
        $this->cdBancoFormulario = $formulario->getCdbanco();
        $this->nrTpMovimentoFormulario = $formulario->getNrtipomoviment();
        $this->nrTipoHierarqOrg = $formulario->getNrhierarqorg();
        $this->dtPagamento = $formulario->getDtpagamento();

        $this->eventoPagamento = $this->evento->retornaEvento($formulario->getNreventom());
        $this->nrEventoPagamento = $formulario->getNreventom();
    }


    public function preparaHolerite($vinculo, $totalizar = false){
        $this->vinculo = $vinculo;
        $this->pessoa = $this->objPessoa->retornaPessoaAtual($this->vinculo->getNrpessoa());
        $this->preparaEstrutura();

            $this->preparaPessoa();
            if ($this->pessoa->banco === $this->cdBancoFormulario) {
                $this->preparaVinculo();
                if($this->estrutura) {
                    switch($this->estrutura->banco) {
                        case HoleriteSantander::CODIGO_BANCO_SANTANDER:
                            $this->holeriteSantander->geraHolerite($this->estrutura, $this->pessoa, $this->vinculo, false);
                        break;
                        default:
                            MsgCalculo::getInstance()->adicionaMensagem('Documento não é gerado para o banco informado.');
                        break;
                    }
                }
            }else{
                MsgCalculo::getInstance()->adicionaMensagem('Documento eletrônico não gerado para Funcionário '.$this->vinculo->getNmvinculom().' por nao ter conta corrente parametrizado.');
            }
            if($totalizar == true) {
                switch($this->estrutura->banco) {
                    case HoleriteSantander::CODIGO_BANCO_SANTANDER:
                        $this->holeriteSantander->geraHolerite($this->estrutura, $this->pessoa, $this->vinculo, true);
                    break;
                }
            }
    }


    private function preparaBancoEstrutura() {
        $contaCorrenteEstrutura = \Teknisa\Entities\Contcorrparc::findOneByNrparcnegocioAndNrorgAndCdbanco($this->estrutura->historico->getNrparcnegocio(), $this->nrorg, $this->cdBancoFormulario);
        if (is_object($contaCorrenteEstrutura) == true) {
            $this->estrutura->banco = $contaCorrenteEstrutura->getCdbanco();
            $this->estrutura->agencia = $contaCorrenteEstrutura->getCdagencia();
            $this->estrutura->contaCorrente = $contaCorrenteEstrutura->getCdcontcorr();
        } else {
            $this->estrutura->banco = null;
            $this->estrutura->agencia = null;
            $this->estrutura->contaCorrente = null;
            MsgCalculo::getInstance()->adicionaMensagem('Parceiro de negocio '.$this->estrutura->historico->getNrparcnegocio().' da estrutura nao possui conta corrente.');
        }
    }

    private function preparaEstrutura() {

        //$movimentacao = $this->estruturacao->getMovimentacaoVinculoPorTipo($this->vinculo->getNrvinculom(), self::TIPO_TRANSFERENCIA_LEGAL);
        $movimentacao = $this->estruturacao->getMovimentacaoVinculoPorTipoHolerite($this->vinculo->getNrvinculom(), self::TIPO_TRANSFERENCIA_LEGAL);

        if (is_object($movimentacao) === true) {
            $this->estrutura = $this->estruturamEntity->retornaEstrutura($movimentacao->getNrestruturam());
        } else 
            $this->estrutura = null;
        
        if ($this->estrutura !== null) 
            $this->preparaBancoEstrutura();
        else{
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$this->vinculo->getNrvinculom().' com problema na movimentação legal. Documento eletrônico não gerado para o Funcionário');
            return;
        }
        
        if ($this->estrutura->historico) {
            $cnpjEstrutura = $this->estrutura->historico->getCdcnpjestrut();
            $cpfEstrutura = $this->estrutura->historico->getCdcpfestrutura();
        } else 
            MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$this->vinculo->getNrvinculom(). ' com problema na movimentação legal.');

        if (!empty($cnpjEstrutura)) 
            $this->estrutura->nrInscricao = $cnpjEstrutura;
        elseif (!empty($cpfEstrutura)) 
            $this->estrutura->nrInscricao = $cpfEstrutura;
        else
            MsgCalculo::getInstance()->adicionaMensagem('Estrutura sem número de inscrição.');
    }


    private function preparaBancoPessoa() {
        $contaCorrentePessoa = $this->contaCorrenteParceiro->retornaContaCorrentePorParceiro($this->pessoa->getNrparcnegocio());
        if (is_object($contaCorrentePessoa) == true) {
            $this->pessoa->banco = $contaCorrentePessoa->getCdbanco();
            $this->pessoa->agencia = $contaCorrentePessoa->getCdagencia();
            $this->pessoa->contaCorrente = $contaCorrentePessoa->getCdcontcorr();
        } else {
            $this->pessoa->banco = null;
            $this->pessoa->agencia = null;
            $this->pessoa->contaCorrente = null;
        }
    }

    private function preparaPessoa(){

        $this->preparaBancoPessoa();
    }


    private function preparaVinculo(){

        $ocupacao = $this->ocupacao->retornaOcupacao($this->vinculo->historico->getNrcargo());
        if (is_object($ocupacao) === true) {
            if (is_object($ocupacao->historico) === true) {
                $this->vinculo->cargo = $ocupacao->historico->getNmocupacaoh();
            } else {
                $this->vinculo->cargo = null;
            }
        } else {
            $this->vinculo->cargo = null;
            MsgCalculo::getInstance()->adicionaMensagem('Nao parametrizada ocupação para o vinculo ' . $this->vinculo->getNrvinculom());
        }

        $this->itemCalculoFolha->retornaItensPorVinculo($this->nrorg, $this->vinculo->getNrvinculom(), $this->nrCalculoFolha);
        $this->vinculo->itensCalculo = $this->itemCalculoFolha->getItems();
    }


    public function modificaSequencia($linha, $sequencia){

        $linha = $this->registroHolerite->alteraDado(
            $linha, (string)$sequencia, (9 -1), 5, RegistroHolerite::CARACTER_ZERO, RegistroHolerite::PREENCHE_ESQUERDA
        );

        return $linha;
    }


    public function modificaLinhasConsolidacao($tipoRegistro, $linha){

        switch($this->cdBancoFormulario) {
            case HoleriteSantander::CODIGO_BANCO_SANTANDER:
                $linha = $this->holeriteSantander->modificaLinhasConsolidacaoSantander($tipoRegistro, $linha);
            break;
        }

        return $linha;
    }


    private function modificaSequenciaArquivo() {
        $this->holeriteMagnetico->modificaSequenciaArquivoRemessa($this->mesCompetencia, $this->nrorg, $this->cdBancoFormulario);
    }


    public function retroceder(){
        $this->holeriteMagnetico->retroceder($this->nrorg, $this->mesCompetencia, $this->cdBancoFormulario, $this->nrTpMovimentoFormulario);
    }


}