<?php
namespace HCM\FPA\Services;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\FPA\Repository\ContaContabilRepository;
use \HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class ContaContabilService {

    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $environment;
    protected $entityManager;
    protected $newCode;
    protected $dataSourceManager;
    protected $factory;

    public function __construct(Environment $environment, HCMParameters $hcmParameters, EntityManager $entityManager, NewCode $newCode, $dataSourceManager, ContaContabilRepository $factory, HCMParameters $hcmparameters) {
        $this->environment             = $environment;
        $this->entityManager           = $entityManager;
        $this->newCode                 = $newCode;
        $this->dataSourceManager       = $dataSourceManager;
        $this->ContaContabilRepository = $factory;
        $this->hcmParameters           = $hcmparameters;
        $this->nrorg                   = $this->hcmParameters->getNrorgTrabalho();
        $this->operador                = $this->hcmParameters->getCdoperador();
        $this->currentTime             = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function geraNovoCodigo($contador, $nrorg){
        try {
            return $this->newCode->getCodeByTableName($contador, $nrorg);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaMaiorConta($row){
        try {
           return $this->ContaContabilRepository->retornaMaiorConta($row['PAI'],$this->nrorg);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaExclusao($row){
        try {
           return $this->ContaContabilRepository->verificaExclusao($row['CDNVCONTCTBL'], $row['CDCONTCTBL'],$row['NRORG']);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function salvarContaContabil($row){
        try {
            if($row['__is_new']){
                $this->validaChaveContaContabil($row);
            }
            $this->validaCodigoReduzido($row);
            $this->setCamposAuditoria($row);
            $row['DTULTATUCCTB'] = $this->currentTime;
            if($this->validaSeContaAnteriorNaoExiste($row['CDCONTANTE'])){
                $row['CDCONTANTE'] = null;
            }
            $dataSet = new DataSet("contctbl", [$row]);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaSeContaAnteriorNaoExiste($cdcontante){
        if(empty( $this->entityManager->getRepository(Repositories::CONTACONTABIL)->findOneBy(array('cdcontctbl' => $cdcontante, 'nrorg' => $this->nrorg)) )){
            return true;
        } else {
            return false;
        }
    }
    
    public function deletarSubContas($row){
        try {
            $dataSet = new DataSet("contctblcorre_grupocontacorre_natucontcorre", [$row]);
            $this->dataSourceManager->delete($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function salvarSubContas($row){
        try {
            if($row['__is_new'])
                $row ['NRSEQUENCIAL'] = $this->geraNovoCodigo('CONTACORRELAT',$this->nrorg);    
            $this->validaSubContas($row);
            $this->setCamposAuditoria($row);
            $dataSet = new DataSet("contctblcorre_grupocontacorre_natucontcorre", [$row]);
            $this->dataSourceManager->persist($dataSet);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function setCamposAuditoria ($row){
        if($row['__is_new']){
            $row['NRORG']          = $this->nrorg;
            $row['IDATIVO']        = 'S';
            $row['DTINCLUSAO']     = $this->currentTime;
            $row['DTULTATU']       = $this->currentTime;
            $row['NRORGINCLUSAO']  = $this->nrorg;
            $row['CDOPERINCLUSAO'] = $this->operador;
            $row['NRORGULTATU']    = $this->nrorg;
            $row['CDOPERULTATU']   = $this->operador;
        }else{
            $row['DTULTATU']       = $this->currentTime;
            $row['NRORGULTATU']    = $this->nrorg;
            $row['CDOPERULTATU']   = $this->operador;    
        }
    }
    
    public function validaSubContas ($row){
        $resul = $this->ContaContabilRepository->validaSubContas($row['CDCONTCTBL'], $row['NRSEQGRUPO'], $row['CDCONTACORRE'], $row['NRSEQNATU']);
        if( count($resul) >0 ){
            if( floatval ($row['NRSEQUENCIAL'] ) !== $resul[0]['NRSEQUENCIAL'] ){
                throw new \Exception('Conta correlata já cadastrada com as parametrizações informadas.', 1);
            }
        }
    }
    
    public function validaChaveContaContabil ($row){
        $resul = $this->ContaContabilRepository->verificaContaContabil($row['CDCONTCTBL'], $this->nrorg);
        if( count($resul) >0 ){
            throw new \Exception('Conta Contábil já cadastrada.', 1);
            
        }   
    }
    
    public function validaCodigoReduzido ($row){
        $resul = $this->ContaContabilRepository->validaCodigoReduzido($row['CDCONTREDU'], $this->nrorg);
        if( count($resul) >0 ){
            if( $resul[0]['CDCONTCTBL']!== $row['CDCONTCTBL'])
                throw new \Exception('Código Reduzido já cadastrado.', 1);
            
        }   
    }

    public function salvarCadastramentoAutomatico($row){
        try {
            switch ($row['TIPOCADASTRO']) {
                case '1':
                    $this->ContaContabilRepository->cadastroAutoIdentificadorPadrao( $row['P_IDPADRREL'], $row['P_CONTAINICIAL'], $row['P_CONTAFINAL'], $this->nrorg );
                    break;
                case '2':
                    $this->ContaContabilRepository->cadastroAutoContaVertical( $row['P_CONTABASE'], $row['P_CONTAINICIAL'], $row['P_CONTAFINAL'], $this->nrorg );
                    break;
                case '3':
                    if( $row['TIPOCONTA'] === 'ECD' )
                        $this->ContaContabilRepository->cadastroAutoContaReferencialEcd( $row['P_CONTANTIGA'], $row['P_CONTNOVA'], $this->nrorg );
                    else if( $row['TIPOCONTA'] === 'AUX' )
                        $this->ContaContabilRepository->cadastroAutoContaReferencialAux( $row['P_CONTANTIGA'], $row['P_CONTNOVA'], $this->nrorg );
                    break;
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function validaContaNivelAnterior ($row){
        if( floatval($row['CDNVCONTCTBL']) > 1 ){
            $resul = $this->ContaContabilRepository->verificaContaContabil($row['CDCONTANTE'], $this->nrorg);
            if( count($resul) === 0 )
                throw new \Exception('Nível anterior da conta contábil não cadastrado.', 1);
        }
    }
    
    public function salvaMascara($mascara){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $this->ContaContabilRepository->salvaMascara($mascara, $nrorg);
    }
    
    public function getMascara () {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        return $this->ContaContabilRepository->getMascara($nrorg);
    }
    
    public function setContaCtbl($contdebito, $descdebito){
        $contctbl = Repositories::CONTACONTABIL;
        $newcontctbldeb = new $contctbl;
        $newcontctbldeb->setCdcontctbl($contdebito);
        $newcontctbldeb->setCdcontante(null);
        $newcontctbldeb->setCdcontctblanali(null);
        $newcontctbldeb->setCdcontplanrefe(null);
        $newcontctbldeb->setCdcontredu($contdebito);
        $newcontctbldeb->setCdcontrefeaux(null);
        $newcontctbldeb->setCdcontrefepre(null);
        $newcontctbldeb->setCdctblinte(null);
        $newcontctbldeb->setCdindplacon(null);
        $newcontctbldeb->setCdnvcontctbl(1);
        $newcontctbldeb->setCdrateio(null);
        $newcontctbldeb->setDtultatucctb(DateUtil::getDataAtual());
        $newcontctbldeb->setIdtipocome(null);
        $newcontctbldeb->setIdtipocont('S');
        $newcontctbldeb->setIdtipopess(null);
        $newcontctbldeb->setNmcontctbl($descdebito);
        
        $newcontctbldeb->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $newcontctbldeb->setDtinclusao(DateUtil::getDataAtual());
        $newcontctbldeb->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $newcontctbldeb->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $newcontctbldeb->setDtultatu(DateUtil::getDataAtual());
        $newcontctbldeb->setCdoperultatu($this->hcmParameters->getCdoperador());
        $newcontctbldeb->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $this->entityManager->persist($newcontctbldeb);
        $this->entityManager->flush($newcontctbldeb);
        return $newcontctbldeb;
    }
    
}