<?php

namespace HCM\FPA\Services;


use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\RateioLancamentoRepository;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\FPA\Util\MemoriaRateio;

class RateioLancamentoService {
    
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, LoteLancEventoService $loteLancEventoService,  HCMParameters $hcmParameters, RateioLancamentoRepository $rateioLancamentoRepository) {
        $this->entityManager = $entityManager;
        $this->loteLancEventoService  = $loteLancEventoService;
        $this->hcmParameters = $hcmParameters;
        $this->rateioLancamentoRepository = $rateioLancamentoRepository;
    }
    
    public function existeItensRateio($nrorg, $nrrateio)
    {
        $existeItemLancamento =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOLANC');
        $existeItemEstrutura =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOESTRUT');
        $existeItemOcupacao = $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOOCUP');
        $existeValores =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOVAL');
        
        if($existeItemLancamento || $existeItemEstrutura || $existeItemOcupacao || $existeValores) {
            return true;
        }

        return false;
    }
    
    public function existeValoresRateio($nrorg, $nrrateio, $dtlancamento)
    {
        $existeValores =  $this->rateioLancamentoRepository->selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOVAL');
        
        if($existeValores) {
            return true;
        }

        return false;
    }

    public function importarCuponsOdhen($nrorg, $nrrateio, $nrestruturam, $cdunidade, $dtinicio, $dtfim)
    {
        try {
            $curl = curl_init();
            
            $urlBase = 'https://odh-sale-api.teknisa.cloud/pobrejuan/index.php';
            $token3469 = 'e+hE6UB4cOVyN5TWZr0pdox+u2/kNsmXEns/uooT2sY8DYAT+II9BfF0Him0PwSATiL6sHaoiCoHY++zpcFmLn4ixIe9z83IeL02a3c7gE/ecLKsrzLWkZTN1R8Zc4ShS0yN5wVyYFpBxHXu42nK1I2I9pPEZFwr59DmuzKSPg+CD2CRrDT9BqWlH7M+UhxsjhjFYq7R2fybUQ0h2jti2YmPOaFCZ82vKPs3jlwCnRYja/R3WevNy0z4wUTj9+xTFruZopONC52Ta5WcV7CMdQ==';
            $codUnidade = $cdunidade;//'0001';
            $dtInicial = $dtinicio;//'01/09/2023';
            $dtFinal = $dtfim;//'08/09/2023';
            
            curl_setopt_array($curl, array(
                CURLOPT_URL => $urlBase.'/getSales',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => '{
                    "subsidiaryCode": "'.$codUnidade.'",
                    "dtInicial": "'.$dtInicial.'",
                    "dtFinal": "'.$dtFinal.'"
                }',
                CURLOPT_HTTPHEADER => array(
                    'Authorization: Bearer '.$token3469,
                    'Content-Type: application/json'
                ),
            ));
            
            $response = json_decode(curl_exec($curl), true);
            
            curl_close($curl);
            
            $qtdLancamentos = 0;
            
            if (isset($response[0])) {
                $response = current($response);
                
                if (!isset($response['sales'])) {
                    throw new \Exception("Erro ao buscar as vendas {".json_encode($response)."}");
                }
            
                $sales = $response['sales'];
                
                foreach ($sales as $sale) {
                    if ($sale['saleStatus'] == 'Concluida') {
                        $cdintegracao = $sale['noteId'];
                        $dtlancamento = DateUtil::getDataDeString($sale['saleDate'], 'd-m-Y H:i:s');
                        $vrlancamento = $sale['saleTotalAmount'];
                        $vracrescimo = $sale['totalAmountAddition'];
                        $vrdesconto = $sale['totalDiscountAmount'];
                        
                        $vrsubtotal1 = $vrvalsubtotal1 = $vrsubtotal2 = $vrvalsubtotal2 = $vrsubtotal3 = $vrvalsubtotal3 = $vrfinal = 0;
                        
                        $this->calculaLancamento(
                            $nrorg, $nrrateio, $vrlancamento, $vracrescimo, $vrdesconto, $vrsubtotal1, $vrvalsubtotal1, $vrsubtotal2, $vrvalsubtotal2, $vrsubtotal3, $vrvalsubtotal3, $vrfinal
                        );
                        
                        $this->addRateiolanc(
                            $nrorg, $nrrateio, $dtlancamento, $vrlancamento, $vracrescimo, $vrdesconto, $vrsubtotal1, $vrsubtotal2, $vrsubtotal3, 
                                  $vrvalsubtotal1, $vrvalsubtotal2, $vrvalsubtotal3, $vrfinal, $cdintegracao
                        );
                        
                        $qtdLancamentos++;
                    }
                }
            } else if (isset($response['success'])) {
                if ($response['success'] == false) {
                    throw new \Exception($response['message']);
                } else {
                    throw new \Exception("Erro ao comunicar com a API {".$response."}");
                }
            } else {
                throw new \Exception("Erro ao comunicar com a API {".$response."}");
            }
            
            return $qtdLancamentos;
        } catch(\Exception $e) {
            throw $e;;
        }  
    }

    public function calculaLancamento($nrorg, $nrrateio, $vrlancamento, $vracrescimo, $vrdesconto, &$vrsubtotal1, &$vrvalsubtotal1, &$vrsubtotal2, &$vrvalsubtotal2, &$vrsubtotal3, &$vrvalsubtotal3, &$vrfinal)
    {   
        $rateio = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg)); 
        
        $vrlancamento = self::aplicaAcrescimoDesconto($vrlancamento, $vracrescimo, $vrdesconto);
        
        $subtotal = null;
        
        if($rateio->getDsoperacao1() != null) {
            $vrsubtotal1 = self::calculaSubTotal($vrlancamento, $rateio->getVrtaxa1()); 
            $subtotal = self::calculaVrFinalLancamento($vrlancamento, $vrsubtotal1, $rateio->getDsoperacao1());
            $vrvalsubtotal1 = $subtotal;
        } else {
            $vrsubtotal1 = null;
            $vrvalsubtotal1 = null;
        }

        if($rateio->getDsoperacao2() != null) {   
            $lancamento = !is_null($subtotal) ? $subtotal : $vrlancamento;
            $vrsubtotal2 = self::calculaSubTotal($lancamento, $rateio->getVrtaxa2()); 
            $subtotal = self::calculaVrFinalLancamento($lancamento, $vrsubtotal2, $rateio->getDsoperacao2());
            $vrvalsubtotal2 = $subtotal;
        } else {
            $vrsubtotal2 = null;
            $vrvalsubtotal2 = null;
        }


        if($rateio->getDsoperacao3() != null) {
            $lancamento = !is_null($subtotal) ? $subtotal : $vrlancamento;
            $vrsubtotal3 = self::calculaSubTotal($lancamento, $rateio->getVrtaxa3()); 
            $subtotal = self::calculaVrFinalLancamento($lancamento, $vrsubtotal3, $rateio->getDsoperacao3());
            $vrvalsubtotal3 = $subtotal;
        } else {
            $vrsubtotal3 = null;
            $vrvalsubtotal3 = null;
        }
        
        if($subtotal == null) {
            $vrfinal = $vrlancamento;
            return;
        }

        $vrfinal = $subtotal;
        return;
    }

    private static function calculaSubTotal($vrlancamento, $taxa)
    {   
        $resultado = ($vrlancamento * ($taxa/100));
        return $resultado;
    }

    private static function calculaVrFinalLancamento($vrlancamento, $vrsubtotal, $operação)
    {   
        if($operação == 'SOMA') {
            $resultado = $vrlancamento + $vrsubtotal;
            return $resultado;
        }

        if($operação == 'SUBTRAI') {
            $resultado = $vrlancamento - $vrsubtotal;
            return $resultado;
        }      
    }

    private static function aplicaAcrescimoDesconto($vrlancamento, $vracrescimo, $vrdesconto)
    {    
        if($vracrescimo != null) {
            $vrlancamento = $vrlancamento + floatval($vracrescimo);
        }

        if($vrdesconto != null) {
            $vrlancamento = $vrlancamento - floatval($vrdesconto);
        }
        
        return $vrlancamento;
    }
    
    public function addRateiolanc($nrorg, $nrrateio, $dtlancamento, $vrlancamento, $vracrescimo, $vrdesconto, $vrsubtotal1, $vrsubtotal2, $vrsubtotal3, 
                                  $vrvalsubtotal1, $vrvalsubtotal2, $vrvalsubtotal3, $vrfinal, $cdintegracao)
    {
        $fpaRateiolanc = Repositories::FPA_RATEIOLANC;
        
        if (!is_null($cdintegracao)) {
            $rateiolanc = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findOneBy(array('nrorg' => $nrorg, 'nrrateio' => $nrrateio, 'cdintegracao' => $cdintegracao));
        } else {
            $rateiolanc = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findOneBy(array('nrorg' => $nrorg, 'nrrateio' => $nrrateio, 'dtlancamento' => $dtlancamento));
        }
        
        if (is_object($rateiolanc)) {
            $rateiolanc->setVrlancamento($vrlancamento);
            $rateiolanc->setVracrescimo($vracrescimo);
            $rateiolanc->setVrdesconto($vrdesconto);
            $rateiolanc->setVrsubtotal1($vrsubtotal1);
            $rateiolanc->setVrsubtotal2($vrsubtotal2);
            $rateiolanc->setVrsubtotal3($vrsubtotal3);
            $rateiolanc->setVrvalsubtotal1($vrvalsubtotal1);
            $rateiolanc->setVrvalsubtotal2($vrvalsubtotal2);
            $rateiolanc->setVrvalsubtotal3($vrvalsubtotal3);
            $rateiolanc->setVrfinal($vrfinal);
            $rateiolanc->setCdintegracao($cdintegracao);
            
            $rateiolanc->setDtultatu(DateUtil::getDataAtual());
            $rateiolanc->setCdoperultatu($this->hcmParameters->getCdoperador());
            $rateiolanc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        } else {
            $rateiolanc = new $fpaRateiolanc;
            $rateiolanc->setNrorg($nrorg);
            $rateiolanc->setNrrateio($nrrateio);
            $rateiolanc->setDtlancamento($dtlancamento);
            $rateiolanc->setVrlancamento($vrlancamento);
            $rateiolanc->setVracrescimo($vracrescimo);
            $rateiolanc->setVrdesconto($vrdesconto);
            $rateiolanc->setVrsubtotal1($vrsubtotal1);
            $rateiolanc->setVrsubtotal2($vrsubtotal2);
            $rateiolanc->setVrsubtotal3($vrsubtotal3);
            $rateiolanc->setVrvalsubtotal1($vrvalsubtotal1);
            $rateiolanc->setVrvalsubtotal2($vrvalsubtotal2);
            $rateiolanc->setVrvalsubtotal3($vrvalsubtotal3);
            $rateiolanc->setVrfinal($vrfinal);
            $rateiolanc->setCdintegracao($cdintegracao);
            $rateiolanc->setIdativo('S');
            
            $rateiolanc->setDtinclusao(DateUtil::getDataAtual());
            $rateiolanc->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $rateiolanc->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $rateiolanc->setDtultatu(DateUtil::getDataAtual());
            $rateiolanc->setCdoperultatu($this->hcmParameters->getCdoperador());
            $rateiolanc->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        }
        
        $rateiolanc->setNrrateiolanc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOLANC', 1, 12, true));
        
        $this->entityManager->persist($rateiolanc);
        $this->entityManager->flush();
        
        return $rateiolanc;
    }
    
    public function replicaParametroRateioEstrutura($row){
        $arrayEstruturasDestino[] = '';
        
        $listaNrrateioEstruturaOrigem = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO_ALVO']));
            
        $listaNrrateioEstruturaDestino = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO']));
        
       
        if(empty($listaNrrateioEstruturaOrigem)){
            return null;
        }
        
        $estruturasClonadas = array();
        if(!empty($listaNrrateioEstruturaDestino)){
            foreach($listaNrrateioEstruturaDestino as $listaEstruturaDestino){
                $arrayEstruturasDestino[] = $listaEstruturaDestino->getNrestruturam();
            }
        }
        foreach($listaNrrateioEstruturaOrigem as $listaEstruturaOrigem){
            if(!in_array($listaEstruturaOrigem->getNrestruturam(),$arrayEstruturasDestino)){
               array_push($estruturasClonadas,$listaEstruturaOrigem);
            }
        }
        if($estruturasClonadas){
            foreach($estruturasClonadas as $clone){
                $auditoria = $this->setAuditoria();
                $fpa_rateioestrut = Repositories::FPA_RATEIOESTRUT;
                $novaListaNrrateioEstrut = new $fpa_rateioestrut;
                $novaListaNrrateioEstrut->setNrorg($auditoria['NRORG']);
                $novaListaNrrateioEstrut->setDtinclusao($auditoria['DTINCLUSAO']);
                $novaListaNrrateioEstrut->setNrestruturam($clone->getNrestruturam());
                $novaListaNrrateioEstrut->setCdoperinclusao($auditoria['CDOPERINCLUSAO']);
                $novaListaNrrateioEstrut->setNrorginclusao($auditoria['NRORGINCLUSAO']);
                $novaListaNrrateioEstrut->setNrrateio($row['NRRATEIO']);
                $novaListaNrrateioEstrut->setNrpeso($clone->getNrpeso());
                $novaListaNrrateioEstrut->setNrrateioestrut(\HCM\Util\NovoCodigo::geraCodigo($row['NRORG'],'FPA_RATEIOESTRUT', 1, 12, true));
                $novaListaNrrateioEstrut->setIdativo('S');
                $novaListaNrrateioEstrut->setVrporcentagem($clone->getVrporcentagem());
                $this->entityManager->persist($novaListaNrrateioEstrut);
                $this->entityManager->flush();
            }
            $this->calculaPorcentagem($auditoria['NRORG'], $row['NRRATEIO'], 'FPA_RATEIOESTRUT');
            return true;
        }
        return false;
    }
    
    public function replicaParametroRateioOcupacao($row){
        
        $arrayOcupacaoDestino[] = '';
        $listaNrrateioOcupacaoOrigem = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO_ALVO']));
            
        $listaNrrateioOcupacaoDestino = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findBy(
            array('nrorg' => $row['NRORG'], 'nrrateio' =>$row['NRRATEIO']));
        
       
        
        if(empty($listaNrrateioOcupacaoOrigem)){
            return null;
        }
        
        $ocupacoesClonadas = array();
        if(!empty($listaNrrateioOcupacaoDestino)){
            foreach($listaNrrateioOcupacaoDestino as $listaOcupacaoDestino){
                $arrayOcupacaoDestino[] = $listaOcupacaoDestino->getNrocupacaom();
            }
        }
        foreach($listaNrrateioOcupacaoOrigem as $listaOcupacaoOrigem){
            if(!in_array($listaOcupacaoOrigem->getNrocupacaom(),$arrayOcupacaoDestino)){
               array_push($ocupacoesClonadas,$listaOcupacaoOrigem);
            }
        }
        
        if($ocupacoesClonadas){
            foreach($ocupacoesClonadas as $clone){
                $auditoria = $this->setAuditoria();
                $fpa_rateioocup = Repositories::FPA_RATEIOOCUP;
                $novaListaNrrateioOcup = new $fpa_rateioocup;
                $novaListaNrrateioOcup->setNrorg($auditoria['NRORG']);
                $novaListaNrrateioOcup->setDtinclusao($auditoria['DTINCLUSAO']);
                $novaListaNrrateioOcup->setNrestruturam($clone->getNrestruturam());
                $novaListaNrrateioOcup->setNrocupacaom($clone->getNrocupacaom());
                $novaListaNrrateioOcup->setCdoperinclusao($auditoria['CDOPERINCLUSAO']);
                $novaListaNrrateioOcup->setNrorginclusao($auditoria['NRORGINCLUSAO']);
                $novaListaNrrateioOcup->setNrrateio($row['NRRATEIO']);
                $novaListaNrrateioOcup->setNrpeso($clone->getNrpeso());
                $novaListaNrrateioOcup->setNrrateioocup(\HCM\Util\NovoCodigo::geraCodigo($row['NRORG'],'FPA_RATEIOOCUP', 1, 12, true));
                $novaListaNrrateioOcup->setIdativo('S');
                $novaListaNrrateioOcup->setVrporcentagem($clone->getVrporcentagem());
                $this->entityManager->persist($novaListaNrrateioOcup);
                $this->entityManager->flush();
            }
            $this->calculaPorcentagem($auditoria['NRORG'], $row['NRRATEIO'], 'FPA_RATEIOOCUP');
            return true;
        }
        return false;
    }
    
    
    public function setAuditoria(){
        $DTINCLUSAO = $this->hcmParameters->getDataAtual();
        $CDOPERINCLUSAO = $this->hcmParameters->getCdoperador();
        $NRORGINCLUSAO = $this->hcmParameters->getNrorgTrabalho();
        $NRORG = $this->hcmParameters->getNrorgTrabalho();
        $auditoria = [
            "DTINCLUSAO" =>  DateUtil::getDataDeString($DTINCLUSAO),
            "CDOPERINCLUSAO" => $CDOPERINCLUSAO,
            "NRORGINCLUSAO" => $NRORGINCLUSAO,
            "NRORG" => $NRORG,
            "DTULTATU" => DateUtil::getDataDeString($DTINCLUSAO),
            "CDOPERULTATU" => $CDOPERINCLUSAO,
            
            
            ]; 
        return $auditoria;
    }

    public function calculoRateioLancamento($nrorg, $nrrateio, $dtinicio, $dtfim, $response)
    {
        try {
            $valores = array();
            $msgRetorno = array();
            $situFuncNaoGeraRateio = null;
            $tipoVincNaoGeraRateio = null;
            $vincEmpregNaoGeraRateio = null;
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            
            $rateio = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg));
            
            $lancamentos = $this->rateioLancamentoRepository->getLancamentos($nrorg, $nrrateio, $dtinicio, $dtfim);
            
            if (empty($lancamentos)) {
                $response->addNotification(new Notification("Não existem lançamentos a serem rateados", Notification::TYPE_ALERT));
                return;
            }
            
            $estrutgerenlist = array();
            $estruturaGerenRateio = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg));
            if(count($estruturaGerenRateio) < 1) {
                $response->addNotification(new Notification("Não foi encontrado cadastro da Estrutura para o rateio", Notification::TYPE_ALERT));
                return;
            } else {
                foreach ($estruturaGerenRateio as $estrutgeren) {
                    array_push($estrutgerenlist, $estrutgeren->getNrestruturam());
                }
            }
        
            $vinculos = $this->rateioLancamentoRepository->getVinculosEstrutLegal($nrorg, $dtmescompetenc, $rateio->getNrestruturam(), $estrutgerenlist, $rateio->getNrsitufuncm(), $rateio->getNrtipovinculom(), $rateio->getNrvinculoempreg());
            if(count($vinculos) < 1) {
                $response->addNotification(new Notification("Não foram encontrados vínculos para o rateio", Notification::TYPE_ALERT));
                return;
            }
            
            $qntdVinculosEstrutGerenCargo = $this->rateioLancamentoRepository->getQntdVinculosPorEstrutGerenCargo($nrorg, $rateio->getNrestruturam(), $rateio->getNrsitufuncm(), $rateio->getNrtipovinculom(), $rateio->getNrvinculoempreg());
            $qntdVinculoSituFuncm = $this->rateioLancamentoRepository->getVinculosEstrutLegal($nrorg, $dtmescompetenc, $rateio->getNrestruturam(), $estrutgerenlist, $rateio->getNrsitufuncm(), $rateio->getNrtipovinculom(), $rateio->getNrvinculoempreg(),true,true,true);
            foreach($qntdVinculoSituFuncm as $situFuncNRateio){
                $situFuncNaoGeraRateio[] = $situFuncNRateio->getNmvinculom();
            }
            $qntdVinculoTipoVinc = $this->rateioLancamentoRepository->getVinculosEstrutLegal($nrorg, $dtmescompetenc, $rateio->getNrestruturam(), $estrutgerenlist, $rateio->getNrsitufuncm(), $rateio->getNrtipovinculom(), $rateio->getNrvinculoempreg(),false,false,true);
            foreach($qntdVinculoTipoVinc as $tipoVincNRateio){
                $tipoVincNaoGeraRateio[] = $tipoVincNRateio->getNmvinculom();
            }
            $qntdVinculoVincEmpreg = $this->rateioLancamentoRepository->getVinculosEstrutLegal($nrorg, $dtmescompetenc, $rateio->getNrestruturam(), $estrutgerenlist, $rateio->getNrsitufuncm(), $rateio->getNrtipovinculom(), $rateio->getNrvinculoempreg(),false,true,false);
            foreach($qntdVinculoVincEmpreg as $vincEmpregNRateio){
                $vincEmpregNaoGeraRateio[] = $vincEmpregNRateio->getNmvinculom();
            }
            $mensagemMemoriaDeCalculo =  MemoriaRateio::getInstance()->mensagemInfoNaoGeraRateio($situFuncNaoGeraRateio,$tipoVincNaoGeraRateio,$vincEmpregNaoGeraRateio);
            foreach ($vinculos as $vinculo) {
                
                $estruturaGerenRateioVinculo = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findOneBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg, 'nrestruturam' => $vinculo->historico->getNrestrutgeren()));
                
                $ocupacaoRateioVinculo = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findOneBy(array('nrrateio' => $nrrateio, 'nrorg' => $nrorg, 'nrestruturam' => $vinculo->historico->getNrestrutgeren(), 'nrocupacaom' => $vinculo->historico->getNrcargo()));
                
                if($estruturaGerenRateioVinculo){
                    $infoEstruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($nrorg,$vinculo->historico->getNrestrutgeren(),DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true));
                }
                if($ocupacaoRateioVinculo){
                    $infoOcupacao = $this->entityManager->getRepository(Repositories::GPE_OCUPACAOH)->findOneBy(array('nrorg' => $nrorg,'nrocupacaom' => $vinculo->historico->getNrcargo()));
                }
                
                if (empty($ocupacaoRateioVinculo)) {
                    $msgRetorno[] = [
                        'NRVINCULOM'  =>  $vinculo->getNrvinculom(),
                        'NMVINCULOM'  => $vinculo->getNmvinculom(),
                        'DSDESCRICAO'  => 'Não foi encontrado cadastro do rateio da Ocupação do vínculo',
                        'IDTIPOMSG' =>  'ATENÇÃO'
                    ];
                    continue;
                }
                
                $qntdVinculosRateio = $this->validaQntdVinculosParaRateio($qntdVinculosEstrutGerenCargo, $estruturaGerenRateioVinculo->getNrestruturam(), $ocupacaoRateioVinculo->getNrocupacaom());
                MemoriaRateio::getInstance()->inicializarDados($qntdVinculosRateio, $estruturaGerenRateioVinculo->getVrporcentagem(), $ocupacaoRateioVinculo->getVrporcentagem() , $vinculo->getNmvinculom(), $vinculo->getNrvinculom(), $infoOcupacao->getNmocupacaoh(), $infoEstruturah->getNmfantasia());
                $mensagemMemoriaDeCalculo .= MemoriaRateio::getInstance()->mensagemInfoVinculo();
                foreach($lancamentos as $lancamento) {
                    $valor['NRORG'] = $nrorg;
                    $valor['NRRATEIO'] = $nrrateio;
                    $valor['NRVINCULOM'] = $vinculo->getNrvinculom();
                    $valor['DTDIALANC'] = DateUtil::getDataDeString($lancamento['DTLANCAMENTO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)->format('d/m/Y');
                    $mensagemMemoriaDeCalculo .= MemoriaRateio::getInstance()->mensagemInfoRateio($valor['DTDIALANC'],$lancamento['VRFINAL']);
                    $valor['VRVALOR'] = $this->calculaRateio($lancamento['VRFINAL'], $estruturaGerenRateioVinculo->getVrporcentagem(), $ocupacaoRateioVinculo->getVrporcentagem(), $qntdVinculosRateio, $response, $valor['DTDIALANC'],$mensagemMemoriaDeCalculo);
                    $valores[] = $valor;
                }
                $mensagemMemoriaDeCalculo .= '______________________________________________________________________________________________________';
            }
            
            if(!empty($msgRetorno)) {
                $response->addDataSet((new \Zeedhi\Framework\DataSource\DataSet('msgRetorno',  $msgRetorno)));
            }
            
            
            return $valores;
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }

    public function validaQntdVinculosParaRateio($qntdVinculosEstrutGerenCargo, $estrutgeren, $ocupacao)
    {
        foreach ($qntdVinculosEstrutGerenCargo as $item) {
            if ($estrutgeren == $item['NRESTRUTGEREN'] && $ocupacao == $item['NRCARGO']) {
                $qntdVinculosRateio = $item['QNTDVINCULOS']; 
                return $qntdVinculosRateio;
            }
        }
        
    }

    public function calculaRateio($vrFinalLancamento, $porcentagemEstrutGeren, $porcentagemOcupacao, $qntdVinculosRateio , $response = null, $dtLanc = null, $mensagemMemoriaDeCalculo = null)
    {
        try {
            
            MemoriaRateio::getInstance()->salvarMsgArquivo($mensagemMemoriaDeCalculo);
            $resultado = ($vrFinalLancamento * ($porcentagemEstrutGeren/100)) * ($porcentagemOcupacao/100);
            $resultado = $resultado/$qntdVinculosRateio;
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('memoriaRateio', array(MemoriaRateio::getInstance()->getCaminhoArquivo())));
            return $resultado;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function calculaPorcentagem($nrorg, $nrrateio, $tabela)
    {
       if($tabela == 'FPA_RATEIOESTRUT') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemEstrutura($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
            $this->rateioLancamentoRepository->setPorcentagemEstrutura($nrorg, $nrrateio, $porcentagem['NRRATEIOESTRUT'],$porcentagem['PERCENT']);
           }
       } 

       if($tabela == 'FPA_RATEIOOCUP') {
           $porcentagens = $this->rateioLancamentoRepository->selectPorcentagemOcupacao($nrorg, $nrrateio);
           foreach($porcentagens as $porcentagem) {
               $this->rateioLancamentoRepository->setPorcentagemOcupacao($nrorg, $nrrateio, $porcentagem['NRRATEIOOCUP'],$porcentagem['PERCENT']);
           }
       }    
   }

    public function exportaLote($itens, $nrorg, $dtmescompetenc, $nreventom)
    {   
        foreach($itens as $item) {
            $itensLote[] = array(
                'NRVINCULOM' => $item['NRVINCULOM'],
                'NMVINCULOM' => '',
                'NREVENTOM' => $nreventom,
                'NMEVENTOH' => '',
                'VRREFFOLHALOTE' => null,
                'VREVENTOLOTE' => $item['VRVALOR']
            );
        }

        $lote = $this->loteLancEventoService->geraLoteLancEvento(
            $nrorg, $dtmescompetenc, 'FPA', 'EXPORTAÇÃO DO RATEIO DE LANÇAMENTO', 'PENDENTE', '', 1,  $itensLote
        );

    }
}