<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;

class AlteSalarioRepository {

    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }



    public function getpisoTetoSalarial($nrorg, $nrvinculom, $dtcompetencia)
    {
        $dtcompetencia = DateUtil::getUltimoDiaDoMes($dtcompetencia)->format('d/m/Y');
        
        $query = 
<<<SQL

     SELECT
        SAL.NROCUPACAOM,
        SAL.VRPISOTABSALARIAL,
        SAL.VRTETOTABSALARIAL,
        OH.NMOCUPACAOH
    FROM
        GSO_TABELASALARIAL SAL
    JOIN GPE_OCUPACAOH      OH ON OH.NRORG = SAL.NRORG
                                 AND OH.NROCUPACAOM = SAL.NROCUPACAOM
                                 AND OH.DTMESCOMPETENC = (
            SELECT
                MAX(DTMESCOMPETENC)
            FROM
                GPE_OCUPACAOH GOH
            WHERE
                    DTMESCOMPETENC <= :P_DTMESCOMPETENC
                AND NRORG = SAL.NRORG
                AND NROCUPACAOM = OH.NROCUPACAOM
        )
    JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = :P_NRVINCULOM
    JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.NRCARGO = SAL.NROCUPACAOM
                        AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) DTMESCOMPETENC
                                          FROM GPE_VINCULOH VHH
                                          WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                          AND VHH.NRCARGO = VH.NRCARGO
                                          AND VHH.NRVINCULOM = VH.NRVINCULOM)
    WHERE
        SAL.NRORG = :P_NRORG
        AND SAL.NROCUPACAOM = VH.NRCARGO

SQL;
            $params = array(
                'P_NRORG'       => $nrorg,
                'P_DTMESCOMPETENC' => $dtcompetencia,
                'P_NRVINCULOM'    => $nrvinculom
            );
            
            $result = $this->connection->fetchAssoc($query, $params);
            return $result;
    }
    
    
    public function getSalarioPorCompetencia($nrorg, $nrvinculom, $dtcompetencia)
    {
        try {
            $dtcompetencia = DateUtil::getUltimoDiaDoMes($dtcompetencia)->format('d/m/Y');
            $query = 
<<<SQL

SELECT *
FROM GPE_ALTESALARIO SAL
WHERE SAL.NRORG = :P_NRORG
 AND SAL.NRVINCULOM = :P_VINCULOM
 AND SAL.DTALTESALARIO = ( SELECT MAX(DTALTESALARIO)
                           FROM GPE_ALTESALARIO SAL2
                           WHERE SAL2.NRORG = :P_NRORG
                            AND SAL2.NRVINCULOM = :P_VINCULOM
                            AND SAL2.DTALTESALARIO <= :P_DTCOMPETENCIA 
                        )
SQL;
            $params = array(
                'P_NRORG'       => $nrorg,
                'P_VINCULOM'    => $nrvinculom,
                'P_DTCOMPETENCIA' => $dtcompetencia,
            );
            
            $result = $this->connection->fetchAssoc($query, $params);
            
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}